﻿using System;
using Ozeki.MediaGateway;

namespace SilverlightSIPClient
{
   /// <summary>
   /// A class what represent a SIPClient and hold the contact with the server.
   /// </summary>
    public class SIPClient
    {
        MediaConnection mediaConnection;
        MediaStreamSender mediaStreamSender;
        MediaStreamReceiver mediaStreamReceiver;
        Microphone microphone;
        AudioPlayer audioPlayer;

        public SIPClient(Microphone microphone, AudioPlayer audioPlayer)
        {
            this.microphone = microphone;
            this.audioPlayer = audioPlayer;

            mediaConnection = new MediaConnection("localhost:4502/SIPGateway");
           // mediaConnection = new MediaConnection("192.168.113.7:4502/SIPGateway");
            mediaConnection.Client = this;
            mediaConnection.ConnectionStateChanged += mediaConnection_ConnectionStateChanged;
        }

        public event EventHandler<GenericEventArgs<ConnectionState>> ConnectionStateChanged;
        public event EventHandler<GenericEventArgs<RegistrationState>> RegistrationStateChanged;
        public event EventHandler<GenericEventArgs<CallState>> CallStateChanged;
        public event EventHandler<GenericEventArgs<CallError>> CallErrorOccurred;
        public event EventHandler<GenericEventArgs<PhoneLineState>> PhoneLineStateChanged;
        public event EventHandler<GenericEventArgs<string>> SIPUserNameReceived;
        public event EventHandler<GenericEventArgs<int>> DTMFReceived;
        public event EventHandler<GenericEventArgs<string>> IncomingCall;
        #region Public Methods

        /// <summary>
        /// Connects to the server.
        /// </summary>
        public void Connect()
        {
            mediaConnection.Connect();
        }

        /// <summary>
        /// Calls a phonenumber.
        /// </summary>
        /// <param name="dialNumber">The dial number.</param>
        public void Call(string dialNumber)
        {
            mediaConnection.InvokeOnConnection("Call", dialNumber);
        }

        /// <summary>
        /// Reject the incoming call or if the call is InCall state ends this call.
        /// </summary>
        public void HangUp()
        {
            mediaConnection.InvokeOnConnection("HangUp");
        }

        /// <summary>
        /// Accepts an incoming call request.
        /// </summary>
        public void Accept()
        {
            mediaConnection.InvokeOnConnection("Accept");
        }

        public void Reject()
        {
            mediaConnection.InvokeOnConnection("Reject");
        }


        public void StartDtmf(int dtmf)
        {
            mediaConnection.InvokeOnConnection("StartDtmf", dtmf);
        }

        public void StopDtmf(int dtmf)
        {
            mediaConnection.InvokeOnConnection("StopDtmf", dtmf);
        }


        /// <summary>
        /// Indicates the call state has changed.
        /// </summary>
        /// <param name="callState">The current state of the call.</param>
        public void OnCallStateChanged(CallState callState)
        {
            if (callState == CallState.Completed)
                CloseStreams();

            var handler = CallStateChanged;

            if(handler != null)
                handler(this, new GenericEventArgs<CallState>(callState));
        }

        /// <summary>
        ///  Indicates an error.
        /// </summary>
        /// <param name="callError"></param>
        public void OnCallErrorOccurred(CallError callError)
        {
            var handler = CallErrorOccurred;

            if(handler != null)
                handler(this, new GenericEventArgs<CallError>(callError));
        }

        /// <summary>
        /// Indicates the SIPClient connection to the server is success or not.
        /// </summary>
        /// <param name="registrationState"></param>
        public void OnRegistrationStateChanged(RegistrationState registrationState)
        {
            var handler = RegistrationStateChanged;

            if(handler != null)
                handler(this, new GenericEventArgs<RegistrationState>(registrationState));
        }

        /// <summary>
        /// Indicates the phoneline registration state has changed.
        /// </summary>
        /// <param name="phoneLineState"></param>
        public void OnPhoneLineStateChanged(PhoneLineState phoneLineState)
        {
            var handler = PhoneLineStateChanged;

            if(handler != null)
                handler(this, new GenericEventArgs<PhoneLineState>(phoneLineState));
        }

        /// <summary>
        /// Starts to send the audio data to the server and starts to play other party's media data.
        /// </summary>
        /// <param name="publisStreamName"></param>
        /// <param name="playStreamName"></param>
        public void OnInCall(string publisStreamName, string playStreamName)
        {
            mediaStreamSender = new MediaStreamSender(mediaConnection);
            mediaStreamSender.AttachMicrophone(microphone);
            mediaStreamSender.Publish(publisStreamName);

            mediaStreamReceiver = new MediaStreamReceiver(mediaConnection);
            mediaStreamReceiver.AttachAudioPlayer(audioPlayer);
            mediaStreamReceiver.Play(playStreamName);
        }


        public void OnIncomingCall(string phoneNumber)
        {
            var handler = IncomingCall;

            if(handler != null)
                handler(this, new GenericEventArgs<string>(phoneNumber));
        }

	/// <summary>
        /// Indicates the SIPClient registered username.
        /// </summary>
        /// <param name="username"></param>
        public void OnRegisteredUserNameReceived(string username)
        {
            var handler = SIPUserNameReceived;
            if (handler != null)
                handler(this, new GenericEventArgs<string>(username));
        }

		/// <summary>
        /// Indicates the incomning DTMF signs.
        /// </summary>
        /// <param name="dtmfSign"></param>
        public void OnDTMFReceived(int dtmfSign)
        {
            var handler = DTMFReceived;
            if (handler != null)
                handler(this, new GenericEventArgs<int>(dtmfSign));
        }


      

       #endregion

        #region Private Methods

        /// <summary>
        ///Indicates the connection state with the server has changed.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void mediaConnection_ConnectionStateChanged(object sender, GenericEventArgs<ConnectionState> e)
        {
            var handler = ConnectionStateChanged;

            if (handler != null)
                handler(this, e);
        }

        void CloseStreams()
        {
            try
            {
                if (mediaStreamReceiver != null)
                {
                    mediaStreamReceiver.Close();
                    mediaStreamReceiver = null;
                }

                if (mediaStreamSender != null)
                {
                    mediaStreamSender.Close();
                    mediaStreamSender = null;
                }
            }
            catch (Exception)
            {
            }
        }

        #endregion
    }
}
